package tech.mercantec.easyeat.ui.profile

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import tech.mercantec.easyeat.ui.auth.WelcomeActivity
import tech.mercantec.easyeat.databinding.FragmentProfileBinding
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.UserInfoResponse
import tech.mercantec.easyeat.helpers.getUserInfo
import tech.mercantec.easyeat.helpers.logout
import kotlin.concurrent.thread

class ProfileFragment : Fragment() {
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val binding = FragmentProfileBinding.inflate(inflater, container, false)

        thread {
            val userInfo: UserInfoResponse
            try {
                userInfo = getUserInfo(requireContext())
            } catch (e: ApiRequestException) {
                activity?.runOnUiThread {
                    Toast.makeText(activity?.applicationContext, e.message, Toast.LENGTH_LONG).show()
                }

                return@thread
            }

            activity?.runOnUiThread {
                binding.username.text = userInfo.userName
                binding.email.text = userInfo.email

                binding.editProfile.setOnClickListener {
                    val intent = Intent(activity, EditProfileActivity::class.java)
                    intent.putExtra("username", userInfo.userName)
                    intent.putExtra("email", userInfo.email)
                    startActivity(intent)
                }
            }
        }

        binding.changePassword.setOnClickListener {
            startActivity(Intent(activity, ChangePasswordActivity::class.java))
        }

        binding.logout.setOnClickListener {
            logout(requireContext())

            val intent = Intent(activity, WelcomeActivity::class.java)
            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
            startActivity(intent)
        }

        return binding.root
    }
}
